/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
****************************************************/
#include "LCD_1in69.h"
#include "DEV_Config.h"

#include <stdlib.h>		//itoa()
#include <stdio.h>

LCD_ATTRIBUTES LCD;


/******************************************************************************
function :	Hardware reset
parameter:
******************************************************************************/
static void LCD_Reset(void)
{
    LCD_RST_1;
    DEV_Delay_ms(100);
    LCD_RST_0;
    DEV_Delay_ms(100);
    LCD_RST_1;
    DEV_Delay_ms(100);
}

/******************************************************************************
function :	send command
parameter:
     Reg : Command register
******************************************************************************/
static void LCD_SendCommand(UBYTE Reg)
{
    LCD_DC_0;
    LCD_CS_0;
    DEV_SPI_WriteByte(Reg);
  // LCD_CS_1;
}

/******************************************************************************
function :	send data
parameter:
    Data : Write data
******************************************************************************/
static void LCD_SendData_8Bit(UBYTE Data)
{
    LCD_DC_1;
    LCD_CS_0;
    DEV_SPI_WriteByte(Data);
  //  LCD_CS_1;
}

/******************************************************************************
function :	send data
parameter:
    Data : Write data
******************************************************************************/
static void LCD_SendData_16Bit(UWORD Data)
{
    LCD_DC_1;
    LCD_CS_0;
    DEV_SPI_WriteByte((Data >> 8) & 0xFF);
    DEV_SPI_WriteByte(Data & 0xFF);
  //  LCD_CS_1;
}

/******************************************************************************
function :	Initialize the lcd register
parameter:
******************************************************************************/
static void LCD_InitReg(void)
{
    LCD_SendCommand(0x3A);
    LCD_SendData_8Bit(0x05);

    LCD_SendCommand(0xB2);
    LCD_SendData_8Bit(0x0C);
    LCD_SendData_8Bit(0x0C);
    LCD_SendData_8Bit(0x00);
    LCD_SendData_8Bit(0x33);
    LCD_SendData_8Bit(0x33);

    LCD_SendCommand(0xB7);  //Gate Control
    LCD_SendData_8Bit(0x35);

    LCD_SendCommand(0xBB);  //VCOM Setting
    LCD_SendData_8Bit(0x32);



    LCD_SendCommand(0xC2);  //VDV and VRH Command Enable
    LCD_SendData_8Bit(0x01);
    LCD_SendCommand(0xC3);  //VRH Set
    LCD_SendData_8Bit(0x15);
    LCD_SendCommand(0xC4);  //VDV Set
    LCD_SendData_8Bit(0x20);

    LCD_SendCommand(0xC6);  //Frame Rate Control in Normal Mode
    LCD_SendData_8Bit(0x0F);
    
    LCD_SendCommand(0xD0);  // Power Control 1
    LCD_SendData_8Bit(0xA4);
    LCD_SendData_8Bit(0xA1);

    LCD_SendCommand(0xE0);  //Positive Voltage Gamma Control
    LCD_SendData_8Bit(0xD0);
    LCD_SendData_8Bit(0x08);
    LCD_SendData_8Bit(0x0E);
    LCD_SendData_8Bit(0x09);
    LCD_SendData_8Bit(0x09);
    LCD_SendData_8Bit(0x05);
    LCD_SendData_8Bit(0x31);
    LCD_SendData_8Bit(0x33);
    LCD_SendData_8Bit(0x48);
    LCD_SendData_8Bit(0x17);
    LCD_SendData_8Bit(0x14);
    LCD_SendData_8Bit(0x15);
    LCD_SendData_8Bit(0x31);
    LCD_SendData_8Bit(0x34);

    LCD_SendCommand(0xE1);  //Negative Voltage Gamma Control
    LCD_SendData_8Bit(0xD0);
    LCD_SendData_8Bit(0x08);
    LCD_SendData_8Bit(0x0E);
    LCD_SendData_8Bit(0x09);
    LCD_SendData_8Bit(0x09);
    LCD_SendData_8Bit(0x05);
    LCD_SendData_8Bit(0x31);
    LCD_SendData_8Bit(0x33);
    LCD_SendData_8Bit(0x48);
    LCD_SendData_8Bit(0x17);
    LCD_SendData_8Bit(0x14);
    LCD_SendData_8Bit(0x15);
    LCD_SendData_8Bit(0x31);
    LCD_SendData_8Bit(0x34);

    LCD_SendCommand(0x21);  //Display Inversion On

    LCD_SendCommand(0x11);  //Sleep Out

    LCD_SendCommand(0x29);  //Display On
}

/********************************************************************************
function:	Set the resolution and scanning method of the screen
parameter:
		Scan_dir:   Scan direction
********************************************************************************/
static void LCD_SetAttributes(vode)
{

    UBYTE MemoryAccessReg = 0x00;

    //Get GRAM and LCD width and height
    if(HORIZONTAL) {
        LCD.HEIGHT	= LCD_HEIGHT;
        LCD.WIDTH   = LCD_WIDTH;
        MemoryAccessReg = 0X70;
    } else {
        LCD.HEIGHT	= LCD_WIDTH;
        LCD.WIDTH   = LCD_HEIGHT;
        MemoryAccessReg = 0X00;
    }

    // Set the read / write scan direction of the frame memory
    LCD_SendCommand(0x36); //MX, MY, RGB mode
    LCD_SendData_8Bit(MemoryAccessReg);	//0x08 set RGB
}

/********************************************************************************
function :	Initialize the lcd
parameter:
********************************************************************************/
void LCD_1in69_Init()
{
    //Turn on the backlight
 //   LCD_BL_1;

    //Hardware reset
    LCD_Reset();

    //Set the resolution and scanning method of the screen
    LCD_SetAttributes();
    
    //Set the initialization register
    LCD_InitReg();
}

/********************************************************************************
function:	Sets the start position and size of the display area
parameter:
		Xstart 	:   X direction Start coordinates
		Ystart  :   Y direction Start coordinates
		Xend    :   X direction end coordinates
		Yend    :   Y direction end coordinates
********************************************************************************/
void LCD_1in69_SetWindows(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend)
{Ystart+=20; Yend+=20;
    //set the X coordinates
    LCD_SendCommand(0x2A);
    LCD_SendData_8Bit((Xstart >> 8) & 0xFF);
    LCD_SendData_8Bit(Xstart & 0xFF);
    LCD_SendData_8Bit(((Xend  - 1) >> 8) & 0xFF);
    LCD_SendData_8Bit((Xend  - 1) & 0xFF);

    //set the Y coordinates
    LCD_SendCommand(0x2B);
    LCD_SendData_8Bit((Ystart >> 8) & 0xFF);
    LCD_SendData_8Bit(Ystart & 0xFF);
    LCD_SendData_8Bit(((Yend  - 1) >> 8) & 0xFF);
    LCD_SendData_8Bit((Yend  - 1) & 0xFF);

    LCD_SendCommand(0X2C);
}

/******************************************************************************
function :	Clear screen
parameter:
******************************************************************************/
void LCD_1in69_Clear(UWORD Color)
{
   UDOUBLE j;
    UWORD Image[LCD_WIDTH*LCD_HEIGHT];
    
    Color = ((Color<<8)&0xff00)|(Color>>8);
   
    for (j = 0; j < LCD_HEIGHT*LCD_WIDTH; j++) {
        Image[j] = Color;
    }
    
    LCD_1in69_SetWindows(0, 0, LCD_WIDTH, LCD_HEIGHT);
    LCD_DC_1;
    for(j = 0; j < LCD_HEIGHT; j++){
        DEV_SPI_Write_nByte((uint8_t *)&Image[j*LCD_WIDTH], LCD_WIDTH*2);
    }
}

/******************************************************************************
function :	Sends the image buffer in RAM to displays
parameter:
******************************************************************************/
void LCD_1in69_Display(UWORD *Image)
{
    UDOUBLE j;
    LCD_1in69_SetWindows(0, 0, LCD_WIDTH, LCD_HEIGHT);
    LCD_DC_1;
    for (j = 0; j < LCD_HEIGHT; j++) {
        DEV_SPI_Write_nByte((uint8_t *)&Image[j*LCD_WIDTH], LCD_WIDTH*2);
    }
}

void LCD_1in69_DisplayWindows(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend, UWORD *Image)
{
    // display
    UDOUBLE Addr = 0;

    UWORD j;
    LCD_1in69_SetWindows(Xstart, Ystart, Xend-1 , Yend-1);
    LCD_DC_1;
    for (j = Ystart; j < Yend - 1; j++) {
        Addr = Xstart + j * LCD_WIDTH ;
        DEV_SPI_Write_nByte((uint8_t *)&Image[Addr], (Xend-Xstart-1)*2);
    }
}

void LCD_1in69_DisplayPoint(UWORD X, UWORD Y, UWORD Color)
{
    LCD_1in69_SetWindows(X,Y,X,Y);
    LCD_SendData_16Bit(Color);
}

void  Handler_1in69_LCD(int signo)
{
    //System Exit
    printf("\r\nHandler:Program stop\r\n");     
    DEV_ModuleExit();
    exit(0);
}
